# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#   Modifications copyright (C) 2022 Cros-Nest
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Interface for the Distribution functionality of CosmosSDK."""

from abc import ABC, abstractmethod

from cosmpy.protos.cosmos.distribution.v1beta1.query_pb2 import (
    QueryCommunityPoolResponse,
    QueryDelegationRewardsRequest,
    QueryDelegationRewardsResponse,
    QueryDelegationTotalRewardsRequest,
    QueryDelegationTotalRewardsResponse,
    QueryDelegatorValidatorsRequest,
    QueryDelegatorValidatorsResponse,
    QueryDelegatorWithdrawAddressRequest,
    QueryDelegatorWithdrawAddressResponse,
    QueryParamsResponse,
    QueryValidatorCommissionRequest,
    QueryValidatorCommissionResponse,
    QueryValidatorOutstandingRewardsRequest,
    QueryValidatorOutstandingRewardsResponse,
    QueryValidatorSlashesRequest,
    QueryValidatorSlashesResponse,
)


class Distribution(ABC):
    """Distribution abstract class."""

    @abstractmethod
    def CommunityPool(self) -> QueryCommunityPoolResponse:
        """
        CommunityPool queries the community pool coins.
        :return: a QueryCommunityPoolResponse instance
        """

    @abstractmethod
    def DelegationTotalRewards(
        self, request: QueryDelegationTotalRewardsRequest
    ) -> QueryDelegationTotalRewardsResponse:
        """
        DelegationTotalRewards queries the total rewards accrued by a each validator.
        :param request: a QueryDelegationTotalRewardsRequest instance
        :return: a QueryDelegationTotalRewardsResponse instance
        """

    @abstractmethod
    def DelegationRewards(
        self, request: QueryDelegationRewardsRequest
    ) -> QueryDelegationRewardsResponse:
        """
        DelegationRewards queries the total rewards accrued by a delegation.
        :param request: a QueryDelegationRewardsRequest instance
        :return: a QueryDelegationRewardsResponse instance
        """

    @abstractmethod
    def DelegatorValidators(
        self, request: QueryDelegatorValidatorsRequest
    ) -> QueryDelegatorValidatorsResponse:
        """
        DelegatorValidators queries the validators of a delegator.
        :param request: a QueryDelegatorValidatorsRequest instance
        :return: a QueryDelegatorValidatorsResponse instance
        """

    @abstractmethod
    def DelegatorWithdrawAddress(
        self, request: QueryDelegatorWithdrawAddressRequest
    ) -> QueryDelegatorWithdrawAddressResponse:
        """
        DelegatorWithdrawAddress queries withdraw address of a delegator.
        :param request: a QueryDelegatorWithdrawAddressRequest instance
        :return: a QueryDelegatorWithdrawAddressResponse instance
        """

    @abstractmethod
    def Params(self) -> QueryParamsResponse:
        """
        Params queries params of the distribution module.
        :return: a QueryParamsResponse instance
        """

    @abstractmethod
    def ValidatorCommission(
        self, request: QueryValidatorCommissionRequest
    ) -> QueryValidatorCommissionResponse:
        """
        ValidatorCommission queries accumulated commission for a validator.
        :param request: QueryValidatorCommissionRequest
        :return: QueryValidatorCommissionResponse
        """

    @abstractmethod
    def ValidatorOutstandingRewards(
        self, request: QueryValidatorOutstandingRewardsRequest
    ) -> QueryValidatorOutstandingRewardsResponse:
        """
        ValidatorOutstandingRewards queries rewards of a validator address.
        :param request: QueryValidatorOutstandingRewardsRequest
        :return: QueryValidatorOutstandingRewardsResponse
        """

    @abstractmethod
    def ValidatorSlashes(
        self, request: QueryValidatorSlashesRequest
    ) -> QueryValidatorSlashesResponse:
        """
        ValidatorSlashes queries slash events of a validator.
        :param request: QueryValidatorSlashesRequest
        :return: QueryValidatorSlashesResponse
        """



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Implementation of IBC Applications Transfer  interface using REST."""
from google.protobuf.json_format import Parse

from cosmpy.common.rest_client import RestClient
from cosmpy.ibc.core.client.interface import IBCCoreClient  # type: ignore
from cosmpy.protos.ibc.core.client.v1.query_pb2 import (
    QueryClientParamsRequest,
    QueryClientParamsResponse,
    QueryClientStateRequest,
    QueryClientStateResponse,
    QueryClientStatesRequest,
    QueryClientStatesResponse,
    QueryConsensusStateRequest,
    QueryConsensusStateResponse,
    QueryConsensusStatesRequest,
    QueryConsensusStatesResponse,
)


class IBCCoreClientRestClient(IBCCoreClient):
    """IBC Core Client REST client."""

    API_URL = "/ibc/core/client/v1beta1"

    def __init__(self, rest_api: RestClient) -> None:
        """
        Initialize.

        :param rest_api: RestClient api
        """
        self._rest_api = rest_api

    def ClientState(self, request: QueryClientStateRequest) -> QueryClientStateResponse:
        """
        ClientState queries an IBC light client.

        :param request: QueryClientStateRequest
        :return: QueryClientStateResponse
        """
        json_response = self._rest_api.get(
            f"{self.API_URL}/client_states/{request.client_id}"
        )
        return Parse(json_response, QueryClientStateResponse())

    def ClientStates(
        self, request: QueryClientStatesRequest
    ) -> QueryClientStatesResponse:
        """
        ClientStates queries all the IBC light clients of a chain.

        :param request: QueryClientStatesRequest
        :return: QueryClientStatesResponse
        """
        json_response = self._rest_api.get(f"{self.API_URL}/client_states", request)
        return Parse(json_response, QueryClientStatesResponse())

    def ConsensusState(
        self, request: QueryConsensusStateRequest
    ) -> QueryConsensusStateResponse:
        """
        ConsensusState queries a consensus state associated with a client state at a given height.

        :param request: QueryConsensusStateRequest
        :return: QueryConsensusStateResponse
        """
        json_response = self._rest_api.get(
            f"{self.API_URL}/consensus_states/{request.client_id}/revision/{request.revision_number}/height/{request.revision_height}"
        )
        return Parse(json_response, QueryConsensusStateResponse())

    def ConsensusStates(
        self, request: QueryConsensusStatesRequest
    ) -> QueryConsensusStatesResponse:
        """
        ConsensusStates queries all the consensus states associated with a given client.

        :param request: QueryConsensusStatesRequest
        :return: QueryConsensusStatesResponse
        """
        json_response = self._rest_api.get(
            f"{self.API_URL}/consensus_states/{request.client_id}", request
        )
        return Parse(json_response, QueryConsensusStatesResponse())

    def ClientParams(
        self, request: QueryClientParamsRequest
    ) -> QueryClientParamsResponse:
        """
        ClientParams queries all parameters of the IBC client.

        :param request: QueryClientParamsRequest
        :return: QueryClientParamsResponse
        """
        json_response = self._rest_api.get("/ibc/client/v1beta1/params")
        return Parse(json_response, QueryClientParamsResponse())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/slashing/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cosmos/slashing/v1beta1/tx.proto',
  package='cosmos.slashing.v1beta1',
  syntax='proto3',
  serialized_options=b'Z-github.com/cosmos/cosmos-sdk/x/slashing/types\250\342\036\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n cosmos/slashing/v1beta1/tx.proto\x12\x17\x63osmos.slashing.v1beta1\x1a\x14gogoproto/gogo.proto\"L\n\tMsgUnjail\x12\x35\n\x0evalidator_addr\x18\x01 \x01(\tB\x1d\xf2\xde\x1f\x0eyaml:\"address\"\xea\xde\x1f\x07\x61\x64\x64ress:\x08\x88\xa0\x1f\x00\x98\xa0\x1f\x01\"\x13\n\x11MsgUnjailResponse2_\n\x03Msg\x12X\n\x06Unjail\x12\".cosmos.slashing.v1beta1.MsgUnjail\x1a*.cosmos.slashing.v1beta1.MsgUnjailResponseB3Z-github.com/cosmos/cosmos-sdk/x/slashing/types\xa8\xe2\x1e\x01\x62\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,])




_MSGUNJAIL = _descriptor.Descriptor(
  name='MsgUnjail',
  full_name='cosmos.slashing.v1beta1.MsgUnjail',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='validator_addr', full_name='cosmos.slashing.v1beta1.MsgUnjail.validator_addr', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\016yaml:\"address\"\352\336\037\007address', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\210\240\037\000\230\240\037\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=83,
  serialized_end=159,
)


_MSGUNJAILRESPONSE = _descriptor.Descriptor(
  name='MsgUnjailResponse',
  full_name='cosmos.slashing.v1beta1.MsgUnjailResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=161,
  serialized_end=180,
)

DESCRIPTOR.message_types_by_name['MsgUnjail'] = _MSGUNJAIL
DESCRIPTOR.message_types_by_name['MsgUnjailResponse'] = _MSGUNJAILRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MsgUnjail = _reflection.GeneratedProtocolMessageType('MsgUnjail', (_message.Message,), {
  'DESCRIPTOR' : _MSGUNJAIL,
  '__module__' : 'cosmos.slashing.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.slashing.v1beta1.MsgUnjail)
  })
_sym_db.RegisterMessage(MsgUnjail)

MsgUnjailResponse = _reflection.GeneratedProtocolMessageType('MsgUnjailResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGUNJAILRESPONSE,
  '__module__' : 'cosmos.slashing.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.slashing.v1beta1.MsgUnjailResponse)
  })
_sym_db.RegisterMessage(MsgUnjailResponse)


DESCRIPTOR._options = None
_MSGUNJAIL.fields_by_name['validator_addr']._options = None
_MSGUNJAIL._options = None

_MSG = _descriptor.ServiceDescriptor(
  name='Msg',
  full_name='cosmos.slashing.v1beta1.Msg',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=182,
  serialized_end=277,
  methods=[
  _descriptor.MethodDescriptor(
    name='Unjail',
    full_name='cosmos.slashing.v1beta1.Msg.Unjail',
    index=0,
    containing_service=None,
    input_type=_MSGUNJAIL,
    output_type=_MSGUNJAILRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MSG)

DESCRIPTOR.services_by_name['Msg'] = _MSG

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Ledger faucet API interface."""
import time
from collections import namedtuple
from typing import Optional, Union

import requests

from cosmpy.aerial.config import NetworkConfig
from cosmpy.crypto.address import Address

CosmosFaucetStatus = namedtuple("CosmosFaucetStatus", ["tx_digest", "status"])


class FaucetApi:
    """Faucet API."""

    MAX_RETRY_ATTEMPTS = 10
    POLL_INTERVAL = 3
    FINAL_WAIT_INTERVAL = 5

    FAUCET_STATUS_PENDING = "pending"  # noqa: F841
    FAUCET_STATUS_PROCESSING = "processing"  # noqa: F841
    FAUCET_STATUS_COMPLETED = "complete"  # noqa: F841
    FAUCET_STATUS_FAILED = "failed"  # noqa: F841

    def __init__(self, net_config: NetworkConfig):
        """
        Init faucet API.

        :param net_config: Ledger network confiuguration.
        """
        if net_config.faucet_url is None:
            raise ValueError("Network config has no faucet url set!")  # pragma: nocover
        self._net_config = net_config

    def _claim_url(self) -> str:
        """
        Get claim url.

        :return: url string
        """
        return f"{self._net_config.faucet_url}/api/v3/claims"

    def _status_uri(self, uid: str) -> str:
        """
        Generates the status URI derived .

        :param uid: claim uid.
        :return: url string
        """
        return f"{self._claim_url()}/{uid}"

    def _try_create_faucet_claim(self, address: str) -> Optional[str]:
        """
        Create a token faucet claim request

        :param address: the address to request funds
        :return: None on failure, otherwise the request uid
        """
        uri = self._claim_url()
        response = requests.post(url=uri, json={"address": address})
        uid = None
        if response.status_code == 200:
            try:
                uid = response.json()["uuid"]
            except KeyError:  # pragma: nocover
                raise ValueError(
                    f"key `uid` not found in response_json={response.json()}"
                )

        return uid

    def _try_check_faucet_claim(self, uid: str) -> Optional[CosmosFaucetStatus]:
        """
        Check the status of a faucet request

        :param uid: The request uid to be checked
        :return: None on failure otherwise a CosmosFaucetStatus for the specified uid
        """
        response = requests.get(self._status_uri(uid))
        if response.status_code != 200:  # pragma: nocover
            return None

        # parse the response
        data = response.json()
        tx_digest = None
        if "txStatus" in data["claim"]:
            tx_digest = data["claim"]["txStatus"]["hash"]

        return CosmosFaucetStatus(
            tx_digest=tx_digest,
            status=data["claim"]["status"],
        )

    def get_wealth(self, address: Union[Address, str]) -> None:
        """
        Get wealth from the faucet for the provided address.

        :param address: the address.
        :raises: RuntimeError of explicit faucet failures
        """
        address = str(address)
        uid = self._try_create_faucet_claim(address)
        if uid is None:  # pragma: nocover
            raise RuntimeError("Unable to create faucet claim")

        retry_attempts = self.MAX_RETRY_ATTEMPTS
        while retry_attempts > 0:
            retry_attempts -= 1

            # lookup status form the claim uid
            status = self._try_check_faucet_claim(uid)
            if status is None:  # pragma: nocover
                raise RuntimeError("Failed to check faucet claim status")

            # if the status is complete
            if status.status == self.FAUCET_STATUS_COMPLETED:
                break

            # if the status is failure
            if (
                status.status != self.FAUCET_STATUS_PENDING
                and status.status != self.FAUCET_STATUS_PROCESSING
            ):  # pragma: nocover
                raise RuntimeError(f"Failed to get wealth for {address}")

            # if the status is incomplete
            time.sleep(self.POLL_INTERVAL)
        if retry_attempts == 0:
            raise ValueError("Faucet claim check timed out!")  # pragma: nocover
        # Wait to ensure that balance is increased on chain
        time.sleep(self.FINAL_WAIT_INTERVAL)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tests for REST client."""

from unittest import TestCase
from unittest.mock import Mock, patch

from requests import Response, Session

from cosmpy.common.rest_client import RestClient


class QueryRestClientTestCase(TestCase):
    """Test case of REST client module."""

    @staticmethod
    @patch("requests.session", spec=Session)
    @patch("cosmpy.common.rest_client.MessageToDict")
    def test_get_pass(messageToDict_mock, session_mock):
        """
        Test get method for the positive result.

        :param messageToDict_mock: mock
        :param session_mock: mock
        """
        rest_address = "some url"
        client = RestClient(rest_address)

        session_mock.assert_called_once_with()
        assert client.rest_address == rest_address

        request_url_path = "/my/weird/url/path"
        resp = Mock(spec=Response)
        resp.status_code = 200
        resp.content = "dfdffdss".encode(encoding="utf8")

        request = "some weird request value"
        request_json = {
            "a": 1,
            "b": ["something", "else"],
            "some_dict": {"x": 1, "y": 2},
        }
        messageToDict_mock.return_value = request_json
        session_mock.return_value.get.return_value = resp
        client.get(request_url_path, request)

        messageToDict_mock.assert_called_once_with(request)

        expected_url = f"{rest_address}{request_url_path}?a=1&b=something&b=else&some_dict.x=1&some_dict.y=2"
        session_mock.return_value.get.assert_called_once_with(url=expected_url)

    @patch("requests.session", spec=Session)
    @patch("cosmpy.common.rest_client.MessageToDict")
    def test_get_error(self, messageToDict_mock, session_mock):
        """
        Test get method for the negative result.

        :param messageToDict_mock: mock
        :param session_mock: mock
        """

        rest_address = "some url"
        client = RestClient(rest_address)

        session_mock.assert_called_once_with()
        assert client.rest_address == rest_address

        request_url_path = "/my/weird/url/path"
        resp = Mock(spec=Response)
        resp.status_code = 400
        resp.content = "dfdffdss".encode(encoding="utf8")

        request = "some weird request value"
        request_json = {"a": 1, "b": "something"}
        messageToDict_mock.return_value = request_json
        session_mock.return_value.get.return_value = resp

        with self.assertRaises(Exception) as context:
            client.get(request_url_path, request)

        messageToDict_mock.assert_called_once_with(request)
        self.assertTrue("Error when sending a GET request" in str(context.exception))

    @staticmethod
    @patch("requests.session", spec=Session)
    @patch("cosmpy.common.rest_client.MessageToDict")
    def test_post_pass(messageToDict_mock, session_mock):
        """
        Test post method for the positive result.

        :param messageToDict_mock: mock
        :param session_mock: mock
        """

        rest_address = "some url"
        client = RestClient(rest_address)

        session_mock.assert_called_once_with()
        assert client.rest_address == rest_address

        request_url_path = "/my/weird/url/path"
        resp = Mock(spec=Response)
        resp.status_code = 200
        resp.content = "dfdffdss".encode(encoding="utf8")

        request = "some weird request value"
        request_json = {"a": 1, "b": "something"}
        messageToDict_mock.return_value = request_json

        session_mock.return_value.post.return_value = resp
        client.post(request_url_path, request)

        messageToDict_mock.assert_called_once_with(request)

        _, kwargs = session_mock.return_value.post.call_args
        headers = kwargs["headers"]
        assert (
            headers.items()
            >= {
                "Content-type": "application/json",
                "Accept": "application/json",
            }.items()
        )
        assert kwargs["url"] == f"{rest_address}{request_url_path}"
        assert kwargs["json"] == request_json

    @patch("requests.session", spec=Session)
    @patch("cosmpy.common.rest_client.MessageToDict")
    def test_post_error(self, messageToDict_mock, session_mock):
        """
        Test post method for the negative result.

        :param messageToDict_mock: mock
        :param session_mock: mock
        """

        rest_address = "some url"
        client = RestClient(rest_address)

        session_mock.assert_called_once_with()
        assert client.rest_address == rest_address

        request_url_path = "/my/weird/url/path"
        resp = Mock(spec=Response)
        resp.status_code = 400
        resp.content = "dfdffdss".encode(encoding="utf8")

        request = "some weird request value"
        request_json = {"a": 1, "b": "something"}
        messageToDict_mock.return_value = request_json

        session_mock.return_value.post.return_value = resp

        with self.assertRaises(Exception) as context:
            client.post(request_url_path, request)

        messageToDict_mock.assert_called_once_with(request)
        self.assertTrue("Error when sending a POST request" in str(context.exception))

    @staticmethod
    @patch("requests.session", spec=Session)
    def test_session_close_on_object_deletion(session_mock):
        """
        Test session close for the positive result.

        :param session_mock: mock
        """

        rest_address = "some url"
        client = RestClient(rest_address)

        session_mock.assert_called_once_with()
        assert client.rest_address == rest_address

        del client
        session_mock.return_value.close.assert_called_once_with()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ibc/core/commitment/v1/commitment.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
import proofs_pb2 as proofs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ibc/core/commitment/v1/commitment.proto',
  package='ibc.core.commitment.v1',
  syntax='proto3',
  serialized_options=b'Z<github.com/cosmos/ibc-go/v2/modules/core/23-commitment/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\'ibc/core/commitment/v1/commitment.proto\x12\x16ibc.core.commitment.v1\x1a\x14gogoproto/gogo.proto\x1a\x0cproofs.proto\" \n\nMerkleRoot\x12\x0c\n\x04hash\x18\x01 \x01(\x0c:\x04\x88\xa0\x1f\x00\"9\n\x0cMerklePrefix\x12)\n\nkey_prefix\x18\x01 \x01(\x0c\x42\x15\xf2\xde\x1f\x11yaml:\"key_prefix\"\"9\n\nMerklePath\x12%\n\x08key_path\x18\x01 \x03(\tB\x13\xf2\xde\x1f\x0fyaml:\"key_path\":\x04\x98\xa0\x1f\x00\"5\n\x0bMerkleProof\x12&\n\x06proofs\x18\x01 \x03(\x0b\x32\x16.ics23.CommitmentProofB>Z<github.com/cosmos/ibc-go/v2/modules/core/23-commitment/typesb\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,proofs__pb2.DESCRIPTOR,])




_MERKLEROOT = _descriptor.Descriptor(
  name='MerkleRoot',
  full_name='ibc.core.commitment.v1.MerkleRoot',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='hash', full_name='ibc.core.commitment.v1.MerkleRoot.hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=103,
  serialized_end=135,
)


_MERKLEPREFIX = _descriptor.Descriptor(
  name='MerklePrefix',
  full_name='ibc.core.commitment.v1.MerklePrefix',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key_prefix', full_name='ibc.core.commitment.v1.MerklePrefix.key_prefix', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\021yaml:\"key_prefix\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=194,
)


_MERKLEPATH = _descriptor.Descriptor(
  name='MerklePath',
  full_name='ibc.core.commitment.v1.MerklePath',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key_path', full_name='ibc.core.commitment.v1.MerklePath.key_path', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\017yaml:\"key_path\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\230\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=196,
  serialized_end=253,
)


_MERKLEPROOF = _descriptor.Descriptor(
  name='MerkleProof',
  full_name='ibc.core.commitment.v1.MerkleProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='proofs', full_name='ibc.core.commitment.v1.MerkleProof.proofs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=255,
  serialized_end=308,
)

_MERKLEPROOF.fields_by_name['proofs'].message_type = proofs__pb2._COMMITMENTPROOF
DESCRIPTOR.message_types_by_name['MerkleRoot'] = _MERKLEROOT
DESCRIPTOR.message_types_by_name['MerklePrefix'] = _MERKLEPREFIX
DESCRIPTOR.message_types_by_name['MerklePath'] = _MERKLEPATH
DESCRIPTOR.message_types_by_name['MerkleProof'] = _MERKLEPROOF
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MerkleRoot = _reflection.GeneratedProtocolMessageType('MerkleRoot', (_message.Message,), {
  'DESCRIPTOR' : _MERKLEROOT,
  '__module__' : 'ibc.core.commitment.v1.commitment_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.commitment.v1.MerkleRoot)
  })
_sym_db.RegisterMessage(MerkleRoot)

MerklePrefix = _reflection.GeneratedProtocolMessageType('MerklePrefix', (_message.Message,), {
  'DESCRIPTOR' : _MERKLEPREFIX,
  '__module__' : 'ibc.core.commitment.v1.commitment_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.commitment.v1.MerklePrefix)
  })
_sym_db.RegisterMessage(MerklePrefix)

MerklePath = _reflection.GeneratedProtocolMessageType('MerklePath', (_message.Message,), {
  'DESCRIPTOR' : _MERKLEPATH,
  '__module__' : 'ibc.core.commitment.v1.commitment_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.commitment.v1.MerklePath)
  })
_sym_db.RegisterMessage(MerklePath)

MerkleProof = _reflection.GeneratedProtocolMessageType('MerkleProof', (_message.Message,), {
  'DESCRIPTOR' : _MERKLEPROOF,
  '__module__' : 'ibc.core.commitment.v1.commitment_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.commitment.v1.MerkleProof)
  })
_sym_db.RegisterMessage(MerkleProof)


DESCRIPTOR._options = None
_MERKLEROOT._options = None
_MERKLEPREFIX.fields_by_name['key_prefix']._options = None
_MERKLEPATH.fields_by_name['key_path']._options = None
_MERKLEPATH._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tests for REST implementation of Staking."""

import json
from unittest import TestCase

from google.protobuf.json_format import ParseDict

from cosmpy.protos.cosmos.staking.v1beta1.query_pb2 import (
    QueryDelegationRequest,
    QueryDelegationResponse,
    QueryDelegatorDelegationsRequest,
    QueryDelegatorDelegationsResponse,
    QueryDelegatorUnbondingDelegationsRequest,
    QueryDelegatorUnbondingDelegationsResponse,
    QueryDelegatorValidatorRequest,
    QueryDelegatorValidatorResponse,
    QueryDelegatorValidatorsRequest,
    QueryDelegatorValidatorsResponse,
    QueryHistoricalInfoRequest,
    QueryHistoricalInfoResponse,
    QueryParamsRequest,
    QueryParamsResponse,
    QueryPoolRequest,
    QueryPoolResponse,
    QueryRedelegationsRequest,
    QueryRedelegationsResponse,
    QueryUnbondingDelegationRequest,
    QueryUnbondingDelegationResponse,
    QueryValidatorDelegationsRequest,
    QueryValidatorDelegationsResponse,
    QueryValidatorRequest,
    QueryValidatorResponse,
    QueryValidatorsRequest,
    QueryValidatorsResponse,
    QueryValidatorUnbondingDelegationsRequest,
    QueryValidatorUnbondingDelegationsResponse,
)
from cosmpy.staking.rest_client import StakingRestClient
from tests.helpers import MockRestClient


class StakingRestClientTestCase(TestCase):
    """Test case for StakingRestClient class."""

    @staticmethod
    def test_Validators():
        """Test Validators method."""
        content = {
            "validators": [
                {
                    "operator_address": "string",
                    "jailed": True,
                    "status": "BOND_STATUS_UNSPECIFIED",
                    "tokens": "123",
                    "delegator_shares": "string",
                    "description": {
                        "moniker": "string",
                        "identity": "string",
                        "website": "string",
                        "security_contact": "string",
                        "details": "string",
                    },
                    "unbonding_height": "1",
                    "unbonding_time": "2021-08-18T11:23:18.208Z",
                    "commission": {
                        "commission_rates": {
                            "rate": "0",
                            "max_rate": "1",
                            "max_change_rate": "1",
                        },
                        "update_time": "2021-08-18T11:23:18.208Z",
                    },
                    "min_self_delegation": "0",
                }
            ],
            "pagination": {"next_key": "", "total": "1"},
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryValidatorsResponse())

        staking = StakingRestClient(mock_client)

        assert staking.Validators(QueryValidatorsRequest()) == expected_response
        assert mock_client.last_base_url == "/cosmos/staking/v1beta1/validators"

    @staticmethod
    def test_Validator():
        """Test Validator method."""

        content = {
            "validator": {
                "operator_address": "string",
                "jailed": True,
                "status": "BOND_STATUS_UNSPECIFIED",
                "tokens": "string",
                "delegator_shares": "string",
                "description": {
                    "moniker": "string",
                    "identity": "string",
                    "website": "string",
                    "security_contact": "string",
                    "details": "string",
                },
                "unbonding_height": "0",
                "unbonding_time": "2021-08-18T10:33:53.339Z",
                "min_self_delegation": "string",
            }
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryValidatorResponse())

        staking = StakingRestClient(mock_client)

        assert (
            staking.Validator(QueryValidatorRequest(validator_addr="validator_addr"))
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/validators/validator_addr"
        )

    @staticmethod
    def test_ValidatorDelegations():
        """Test ValidatorDelegations method."""

        content = {
            "delegation_responses": [
                {
                    "delegation": {
                        "delegator_address": "fetchdelegator",
                        "validator_address": "fetchvalidator",
                        "shares": "123",
                    },
                    "balance": {"denom": "atestfet", "amount": "12345"},
                }
            ],
            "pagination": {"next_key": "", "total": "0"},
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryValidatorDelegationsResponse())

        staking = StakingRestClient(mock_client)

        assert (
            staking.ValidatorDelegations(
                QueryValidatorDelegationsRequest(validator_addr="validator_addr")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/validators/validator_addr/delegations"
        )

    @staticmethod
    def test_ValidatorUnbondingDelegations():
        """Test ValidatorUnbondingDelegations method."""

        content = {
            "unbonding_responses": [
                {
                    "delegator_address": "fetchddelegator",
                    "validator_address": "fetchvalidator",
                    "entries": [
                        {
                            "creation_height": "123",
                            "completion_time": "2021-08-18T11:42:45.657Z",
                            "initial_balance": "12",
                            "balance": "123",
                        }
                    ],
                }
            ],
            "pagination": {"next_key": "", "total": "0"},
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(
            content, QueryValidatorUnbondingDelegationsResponse()
        )

        staking = StakingRestClient(mock_client)

        assert (
            staking.ValidatorUnbondingDelegations(
                QueryValidatorUnbondingDelegationsRequest(
                    validator_addr="validator_addr"
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/validators/validator_addr/unbonding_delegations"
        )

    @staticmethod
    def test_Delegation():
        """Test Delegation method."""

        content = {
            "delegation_response": {
                "delegation": {
                    "delegator_address": "fetchdelegator",
                    "validator_address": "fetchvalidator",
                    "shares": "123",
                },
                "balance": {"denom": "atestfet", "amount": "123"},
            }
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryDelegationResponse())

        staking = StakingRestClient(mock_client)

        assert (
            staking.Delegation(
                QueryDelegationRequest(
                    validator_addr="validator_addr", delegator_addr="delegator_addr"
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/validators/validator_addr/delegations/delegator_addr"
        )

    @staticmethod
    def test_UnbondingDelegation():
        """Test UnbondingDelegation method."""

        content = {
            "unbond": {
                "delegator_address": "fetchdelegator",
                "validator_address": "fetchvalidator",
                "entries": [
                    {
                        "creation_height": "1",
                        "completion_time": "2021-08-18T11:55:13.614Z",
                        "initial_balance": "123",
                        "balance": "1234",
                    }
                ],
            }
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryUnbondingDelegationResponse())

        staking = StakingRestClient(mock_client)

        assert (
            staking.UnbondingDelegation(
                QueryUnbondingDelegationRequest(
                    validator_addr="validator_addr", delegator_addr="delegator_addr"
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/validators/validator_addr/delegations/delegator_addr/unbonding_delegation"
        )

    @staticmethod
    def test_DelegatorDelegations():
        """Test DelegatorDelegations method."""

        content = {
            "delegation_responses": [
                {
                    "delegation": {
                        "delegator_address": "fetchdelegator",
                        "validator_address": "fetchvalidator",
                        "shares": "123",
                    },
                    "balance": {"denom": "atestfet", "amount": "123"},
                }
            ],
            "pagination": {"next_key": "", "total": "0"},
        }
        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryDelegatorDelegationsResponse())

        staking = StakingRestClient(mock_client)

        assert (
            staking.DelegatorDelegations(
                QueryDelegatorDelegationsRequest(delegator_addr="delegator_addr")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/delegations/delegator_addr"
        )

    @staticmethod
    def test_DelegatorUnbondingDelegations():
        """Test DelegatorUnbondingDelegations method."""

        content = {
            "unbonding_responses": [
                {
                    "delegator_address": "fetchdelegator",
                    "validator_address": "fetchvalidator",
                    "entries": [
                        {
                            "creation_height": "123",
                            "completion_time": "2021-08-18T12:07:23.832Z",
                            "initial_balance": "123",
                            "balance": "1234",
                        }
                    ],
                }
            ],
            "pagination": {"next_key": "", "total": "0"},
        }

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(
            content, QueryDelegatorUnbondingDelegationsResponse()
        )

        staking = StakingRestClient(mock_client)

        assert (
            staking.DelegatorUnbondingDelegations(
                QueryDelegatorUnbondingDelegationsRequest(
                    delegator_addr="delegator_addr"
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/delegators/delegator_addr/unbonding_delegations"
        )

    @staticmethod
    def test_Redelegations():
        """Test Redelegations method."""

        content = {
            "redelegation_responses": [
                {
                    "redelegation": {
                        "delegator_address": "fetchdelegator",
                        "validator_src_address": "fetchsrc",
                        "validator_dst_address": "fetchdst",
                        "entries": [
                            {
                                "creation_height": "123",
                                "completion_time": "2021-08-18T12:10:21.412Z",
                                "initial_balance": "123",
                                "shares_dst": "123",
                            }
                        ],
                    },
                    "entries": [
                        {
                            "redelegation_entry": {
                                "creation_height": "123",
                                "completion_time": "2021-08-18T12:10:21.412Z",
                                "initial_balance": "123",
                                "shares_dst": "123",
                            },
                            "balance": "123",
                        }
                    ],
                }
            ],
            "pagination": {"next_key": "", "total": "0"},
        }

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryRedelegationsResponse())

        staking = StakingRestClient(mock_client)

        assert (
            staking.Redelegations(
                QueryRedelegationsRequest(delegator_addr="delegator_addr")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/delegators/delegator_addr/redelegations"
        )

    @staticmethod
    def test_DelegatorValidators():
        """Test DelegatorValidators method."""

        content = {
            "validators": [
                {
                    "operator_address": "fetchoperator",
                    "jailed": True,
                    "status": "BOND_STATUS_UNSPECIFIED",
                    "tokens": "123",
                    "delegator_shares": "123",
                    "description": {
                        "moniker": "string",
                        "identity": "string",
                        "website": "string",
                        "security_contact": "string",
                        "details": "string",
                    },
                    "unbonding_height": "123",
                    "unbonding_time": "2021-08-18T12:14:55.780Z",
                    "commission": {
                        "commission_rates": {
                            "rate": "123",
                            "max_rate": "1234",
                            "max_change_rate": "1234",
                        },
                        "update_time": "2021-08-18T12:14:55.780Z",
                    },
                    "min_self_delegation": "123",
                }
            ],
            "pagination": {"next_key": "", "total": "0"},
        }

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryDelegatorValidatorsResponse())

        staking = StakingRestClient(mock_client)

        assert (
            staking.DelegatorValidators(
                QueryDelegatorValidatorsRequest(delegator_addr="delegator_addr")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/delegators/delegator_addr/validators"
        )

    @staticmethod
    def test_DelegatorValidator():
        """Test DelegatorValidator method."""

        content = {
            "validator": {
                "operator_address": "fetchoperator",
                "jailed": True,
                "status": "BOND_STATUS_UNSPECIFIED",
                "tokens": "123",
                "delegator_shares": "123",
                "description": {
                    "moniker": "string",
                    "identity": "string",
                    "website": "string",
                    "security_contact": "string",
                    "details": "string",
                },
                "unbonding_height": "123",
                "unbonding_time": "2021-08-18T12:17:33.141Z",
                "commission": {
                    "commission_rates": {
                        "rate": "123",
                        "max_rate": "1234",
                        "max_change_rate": "1234",
                    },
                    "update_time": "2021-08-18T12:17:33.141Z",
                },
                "min_self_delegation": "123",
            }
        }

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryDelegatorValidatorResponse())

        staking = StakingRestClient(mock_client)

        assert (
            staking.DelegatorValidator(
                QueryDelegatorValidatorRequest(
                    validator_addr="validator_addr", delegator_addr="delegator_addr"
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/staking/v1beta1/delegators/delegator_addr/validators/validator_addr"
        )

    @staticmethod
    def test_HistoricalInfo():
        """Test HistoricalInfo method."""

        content = {
            "hist": {
                "header": {
                    "version": {"block": "123", "app": "123"},
                    "chain_id": "string",
                    "height": "123",
                    "time": "2021-08-18T12:19:32.225Z",
                    "last_block_id": {
                        "hash": "string",
                        "part_set_header": {"total": 0, "hash": "string"},
                    },
                    "last_commit_hash": "string",
                    "data_hash": "string",
                    "validators_hash": "string",
                    "next_validators_hash": "string",
                    "consensus_hash": "string",
                    "app_hash": "string",
                    "last_results_hash": "string",
                    "evidence_hash": "string",
                    "proposer_address": "string",
                },
                "valset": [
                    {
                        "operator_address": "fetchoperator",
                        "jailed": True,
                        "status": "BOND_STATUS_UNSPECIFIED",
                        "tokens": "123",
                        "delegator_shares": "123",
                        "description": {
                            "moniker": "string",
                            "identity": "string",
                            "website": "string",
                            "security_contact": "string",
                            "details": "string",
                        },
                        "unbonding_height": "123",
                        "unbonding_time": "2021-08-18T12:19:32.225Z",
                        "commission": {
                            "commission_rates": {
                                "rate": "123",
                                "max_rate": "1234",
                                "max_change_rate": "1234",
                            },
                            "update_time": "2021-08-18T12:19:32.225Z",
                        },
                        "min_self_delegation": "123",
                    }
                ],
            }
        }

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryHistoricalInfoResponse())

        staking = StakingRestClient(mock_client)

        assert (
            staking.HistoricalInfo(QueryHistoricalInfoRequest(height=1))
            == expected_response
        )
        assert mock_client.last_base_url == "/cosmos/staking/v1beta1/historical_info/1"

    @staticmethod
    def test_Pool():
        """Test Pool method."""

        content = {"pool": {"not_bonded_tokens": "123", "bonded_tokens": "123"}}

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryPoolResponse())

        staking = StakingRestClient(mock_client)

        assert staking.Pool(QueryPoolRequest()) == expected_response
        assert mock_client.last_base_url == "/cosmos/staking/v1beta1/pool"

    @staticmethod
    def test_Params():
        """Test Params method."""

        content = {
            "params": {
                "unbonding_time": "123s",
                "max_validators": 0,
                "max_entries": 0,
                "historical_entries": 0,
                "bond_denom": "atestfet",
            }
        }

        mock_client = MockRestClient(json.dumps(content))

        expected_response = ParseDict(content, QueryParamsResponse())

        staking = StakingRestClient(mock_client)

        assert staking.Params(QueryParamsRequest()) == expected_response
        assert mock_client.last_base_url == "/cosmos/staking/v1beta1/params"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/feegrant/v1beta1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos.feegrant.v1beta1 import feegrant_pb2 as cosmos_dot_feegrant_dot_v1beta1_dot_feegrant__pb2
from cosmos.base.query.v1beta1 import pagination_pb2 as cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cosmos/feegrant/v1beta1/query.proto',
  package='cosmos.feegrant.v1beta1',
  syntax='proto3',
  serialized_options=b'Z\'github.com/cosmos/cosmos-sdk/x/feegrant',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n#cosmos/feegrant/v1beta1/query.proto\x12\x17\x63osmos.feegrant.v1beta1\x1a&cosmos/feegrant/v1beta1/feegrant.proto\x1a*cosmos/base/query/v1beta1/pagination.proto\x1a\x1cgoogle/api/annotations.proto\"9\n\x15QueryAllowanceRequest\x12\x0f\n\x07granter\x18\x01 \x01(\t\x12\x0f\n\x07grantee\x18\x02 \x01(\t\"K\n\x16QueryAllowanceResponse\x12\x31\n\tallowance\x18\x01 \x01(\x0b\x32\x1e.cosmos.feegrant.v1beta1.Grant\"e\n\x16QueryAllowancesRequest\x12\x0f\n\x07grantee\x18\x01 \x01(\t\x12:\n\npagination\x18\x02 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest\"\x8a\x01\n\x17QueryAllowancesResponse\x12\x32\n\nallowances\x18\x01 \x03(\x0b\x32\x1e.cosmos.feegrant.v1beta1.Grant\x12;\n\npagination\x18\x02 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse2\xdf\x02\n\x05Query\x12\xac\x01\n\tAllowance\x12..cosmos.feegrant.v1beta1.QueryAllowanceRequest\x1a/.cosmos.feegrant.v1beta1.QueryAllowanceResponse\">\x82\xd3\xe4\x93\x02\x38\x12\x36/cosmos/feegrant/v1beta1/allowance/{granter}/{grantee}\x12\xa6\x01\n\nAllowances\x12/.cosmos.feegrant.v1beta1.QueryAllowancesRequest\x1a\x30.cosmos.feegrant.v1beta1.QueryAllowancesResponse\"5\x82\xd3\xe4\x93\x02/\x12-/cosmos/feegrant/v1beta1/allowances/{grantee}B)Z\'github.com/cosmos/cosmos-sdk/x/feegrantb\x06proto3'
  ,
  dependencies=[cosmos_dot_feegrant_dot_v1beta1_dot_feegrant__pb2.DESCRIPTOR,cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])




_QUERYALLOWANCEREQUEST = _descriptor.Descriptor(
  name='QueryAllowanceRequest',
  full_name='cosmos.feegrant.v1beta1.QueryAllowanceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='granter', full_name='cosmos.feegrant.v1beta1.QueryAllowanceRequest.granter', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='grantee', full_name='cosmos.feegrant.v1beta1.QueryAllowanceRequest.grantee', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=178,
  serialized_end=235,
)


_QUERYALLOWANCERESPONSE = _descriptor.Descriptor(
  name='QueryAllowanceResponse',
  full_name='cosmos.feegrant.v1beta1.QueryAllowanceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='allowance', full_name='cosmos.feegrant.v1beta1.QueryAllowanceResponse.allowance', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=237,
  serialized_end=312,
)


_QUERYALLOWANCESREQUEST = _descriptor.Descriptor(
  name='QueryAllowancesRequest',
  full_name='cosmos.feegrant.v1beta1.QueryAllowancesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='grantee', full_name='cosmos.feegrant.v1beta1.QueryAllowancesRequest.grantee', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pagination', full_name='cosmos.feegrant.v1beta1.QueryAllowancesRequest.pagination', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=314,
  serialized_end=415,
)


_QUERYALLOWANCESRESPONSE = _descriptor.Descriptor(
  name='QueryAllowancesResponse',
  full_name='cosmos.feegrant.v1beta1.QueryAllowancesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='allowances', full_name='cosmos.feegrant.v1beta1.QueryAllowancesResponse.allowances', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pagination', full_name='cosmos.feegrant.v1beta1.QueryAllowancesResponse.pagination', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=418,
  serialized_end=556,
)

_QUERYALLOWANCERESPONSE.fields_by_name['allowance'].message_type = cosmos_dot_feegrant_dot_v1beta1_dot_feegrant__pb2._GRANT
_QUERYALLOWANCESREQUEST.fields_by_name['pagination'].message_type = cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2._PAGEREQUEST
_QUERYALLOWANCESRESPONSE.fields_by_name['allowances'].message_type = cosmos_dot_feegrant_dot_v1beta1_dot_feegrant__pb2._GRANT
_QUERYALLOWANCESRESPONSE.fields_by_name['pagination'].message_type = cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2._PAGERESPONSE
DESCRIPTOR.message_types_by_name['QueryAllowanceRequest'] = _QUERYALLOWANCEREQUEST
DESCRIPTOR.message_types_by_name['QueryAllowanceResponse'] = _QUERYALLOWANCERESPONSE
DESCRIPTOR.message_types_by_name['QueryAllowancesRequest'] = _QUERYALLOWANCESREQUEST
DESCRIPTOR.message_types_by_name['QueryAllowancesResponse'] = _QUERYALLOWANCESRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

QueryAllowanceRequest = _reflection.GeneratedProtocolMessageType('QueryAllowanceRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYALLOWANCEREQUEST,
  '__module__' : 'cosmos.feegrant.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.feegrant.v1beta1.QueryAllowanceRequest)
  })
_sym_db.RegisterMessage(QueryAllowanceRequest)

QueryAllowanceResponse = _reflection.GeneratedProtocolMessageType('QueryAllowanceResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYALLOWANCERESPONSE,
  '__module__' : 'cosmos.feegrant.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.feegrant.v1beta1.QueryAllowanceResponse)
  })
_sym_db.RegisterMessage(QueryAllowanceResponse)

QueryAllowancesRequest = _reflection.GeneratedProtocolMessageType('QueryAllowancesRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYALLOWANCESREQUEST,
  '__module__' : 'cosmos.feegrant.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.feegrant.v1beta1.QueryAllowancesRequest)
  })
_sym_db.RegisterMessage(QueryAllowancesRequest)

QueryAllowancesResponse = _reflection.GeneratedProtocolMessageType('QueryAllowancesResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYALLOWANCESRESPONSE,
  '__module__' : 'cosmos.feegrant.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.feegrant.v1beta1.QueryAllowancesResponse)
  })
_sym_db.RegisterMessage(QueryAllowancesResponse)


DESCRIPTOR._options = None

_QUERY = _descriptor.ServiceDescriptor(
  name='Query',
  full_name='cosmos.feegrant.v1beta1.Query',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=559,
  serialized_end=910,
  methods=[
  _descriptor.MethodDescriptor(
    name='Allowance',
    full_name='cosmos.feegrant.v1beta1.Query.Allowance',
    index=0,
    containing_service=None,
    input_type=_QUERYALLOWANCEREQUEST,
    output_type=_QUERYALLOWANCERESPONSE,
    serialized_options=b'\202\323\344\223\0028\0226/cosmos/feegrant/v1beta1/allowance/{granter}/{grantee}',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Allowances',
    full_name='cosmos.feegrant.v1beta1.Query.Allowances',
    index=1,
    containing_service=None,
    input_type=_QUERYALLOWANCESREQUEST,
    output_type=_QUERYALLOWANCESRESPONSE,
    serialized_options=b'\202\323\344\223\002/\022-/cosmos/feegrant/v1beta1/allowances/{grantee}',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_QUERY)

DESCRIPTOR.services_by_name['Query'] = _QUERY

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/authz/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from cosmos.base.abci.v1beta1 import abci_pb2 as cosmos_dot_base_dot_abci_dot_v1beta1_dot_abci__pb2
from cosmos.authz.v1beta1 import authz_pb2 as cosmos_dot_authz_dot_v1beta1_dot_authz__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cosmos/authz/v1beta1/tx.proto',
  package='cosmos.authz.v1beta1',
  syntax='proto3',
  serialized_options=b'Z$github.com/cosmos/cosmos-sdk/x/authz\310\341\036\000',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1d\x63osmos/authz/v1beta1/tx.proto\x12\x14\x63osmos.authz.v1beta1\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x14gogoproto/gogo.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/protobuf/any.proto\x1a#cosmos/base/abci/v1beta1/abci.proto\x1a cosmos/authz/v1beta1/authz.proto\"^\n\x08MsgGrant\x12\x0f\n\x07granter\x18\x01 \x01(\t\x12\x0f\n\x07grantee\x18\x02 \x01(\t\x12\x30\n\x05grant\x18\x03 \x01(\x0b\x32\x1b.cosmos.authz.v1beta1.GrantB\x04\xc8\xde\x1f\x00\"\"\n\x0fMsgExecResponse\x12\x0f\n\x07results\x18\x01 \x03(\x0c\"`\n\x07MsgExec\x12\x0f\n\x07grantee\x18\x01 \x01(\t\x12\x44\n\x04msgs\x18\x02 \x03(\x0b\x32\x14.google.protobuf.AnyB \xca\xb4-\x1csdk.Msg, authz.Authorization\"\x12\n\x10MsgGrantResponse\"C\n\tMsgRevoke\x12\x0f\n\x07granter\x18\x01 \x01(\t\x12\x0f\n\x07grantee\x18\x02 \x01(\t\x12\x14\n\x0cmsg_type_url\x18\x03 \x01(\t\"\x13\n\x11MsgRevokeResponse2\xf8\x01\n\x03Msg\x12O\n\x05Grant\x12\x1e.cosmos.authz.v1beta1.MsgGrant\x1a&.cosmos.authz.v1beta1.MsgGrantResponse\x12L\n\x04\x45xec\x12\x1d.cosmos.authz.v1beta1.MsgExec\x1a%.cosmos.authz.v1beta1.MsgExecResponse\x12R\n\x06Revoke\x12\x1f.cosmos.authz.v1beta1.MsgRevoke\x1a\'.cosmos.authz.v1beta1.MsgRevokeResponseB*Z$github.com/cosmos/cosmos-sdk/x/authz\xc8\xe1\x1e\x00\x62\x06proto3'
  ,
  dependencies=[cosmos__proto_dot_cosmos__pb2.DESCRIPTOR,gogoproto_dot_gogo__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,cosmos_dot_base_dot_abci_dot_v1beta1_dot_abci__pb2.DESCRIPTOR,cosmos_dot_authz_dot_v1beta1_dot_authz__pb2.DESCRIPTOR,])




_MSGGRANT = _descriptor.Descriptor(
  name='MsgGrant',
  full_name='cosmos.authz.v1beta1.MsgGrant',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='granter', full_name='cosmos.authz.v1beta1.MsgGrant.granter', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='grantee', full_name='cosmos.authz.v1beta1.MsgGrant.grantee', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='grant', full_name='cosmos.authz.v1beta1.MsgGrant.grant', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=235,
  serialized_end=329,
)


_MSGEXECRESPONSE = _descriptor.Descriptor(
  name='MsgExecResponse',
  full_name='cosmos.authz.v1beta1.MsgExecResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='results', full_name='cosmos.authz.v1beta1.MsgExecResponse.results', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=331,
  serialized_end=365,
)


_MSGEXEC = _descriptor.Descriptor(
  name='MsgExec',
  full_name='cosmos.authz.v1beta1.MsgExec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='grantee', full_name='cosmos.authz.v1beta1.MsgExec.grantee', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msgs', full_name='cosmos.authz.v1beta1.MsgExec.msgs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\312\264-\034sdk.Msg, authz.Authorization', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=367,
  serialized_end=463,
)


_MSGGRANTRESPONSE = _descriptor.Descriptor(
  name='MsgGrantResponse',
  full_name='cosmos.authz.v1beta1.MsgGrantResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=465,
  serialized_end=483,
)


_MSGREVOKE = _descriptor.Descriptor(
  name='MsgRevoke',
  full_name='cosmos.authz.v1beta1.MsgRevoke',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='granter', full_name='cosmos.authz.v1beta1.MsgRevoke.granter', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='grantee', full_name='cosmos.authz.v1beta1.MsgRevoke.grantee', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg_type_url', full_name='cosmos.authz.v1beta1.MsgRevoke.msg_type_url', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=485,
  serialized_end=552,
)


_MSGREVOKERESPONSE = _descriptor.Descriptor(
  name='MsgRevokeResponse',
  full_name='cosmos.authz.v1beta1.MsgRevokeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=554,
  serialized_end=573,
)

_MSGGRANT.fields_by_name['grant'].message_type = cosmos_dot_authz_dot_v1beta1_dot_authz__pb2._GRANT
_MSGEXEC.fields_by_name['msgs'].message_type = google_dot_protobuf_dot_any__pb2._ANY
DESCRIPTOR.message_types_by_name['MsgGrant'] = _MSGGRANT
DESCRIPTOR.message_types_by_name['MsgExecResponse'] = _MSGEXECRESPONSE
DESCRIPTOR.message_types_by_name['MsgExec'] = _MSGEXEC
DESCRIPTOR.message_types_by_name['MsgGrantResponse'] = _MSGGRANTRESPONSE
DESCRIPTOR.message_types_by_name['MsgRevoke'] = _MSGREVOKE
DESCRIPTOR.message_types_by_name['MsgRevokeResponse'] = _MSGREVOKERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MsgGrant = _reflection.GeneratedProtocolMessageType('MsgGrant', (_message.Message,), {
  'DESCRIPTOR' : _MSGGRANT,
  '__module__' : 'cosmos.authz.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.authz.v1beta1.MsgGrant)
  })
_sym_db.RegisterMessage(MsgGrant)

MsgExecResponse = _reflection.GeneratedProtocolMessageType('MsgExecResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGEXECRESPONSE,
  '__module__' : 'cosmos.authz.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.authz.v1beta1.MsgExecResponse)
  })
_sym_db.RegisterMessage(MsgExecResponse)

MsgExec = _reflection.GeneratedProtocolMessageType('MsgExec', (_message.Message,), {
  'DESCRIPTOR' : _MSGEXEC,
  '__module__' : 'cosmos.authz.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.authz.v1beta1.MsgExec)
  })
_sym_db.RegisterMessage(MsgExec)

MsgGrantResponse = _reflection.GeneratedProtocolMessageType('MsgGrantResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGGRANTRESPONSE,
  '__module__' : 'cosmos.authz.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.authz.v1beta1.MsgGrantResponse)
  })
_sym_db.RegisterMessage(MsgGrantResponse)

MsgRevoke = _reflection.GeneratedProtocolMessageType('MsgRevoke', (_message.Message,), {
  'DESCRIPTOR' : _MSGREVOKE,
  '__module__' : 'cosmos.authz.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.authz.v1beta1.MsgRevoke)
  })
_sym_db.RegisterMessage(MsgRevoke)

MsgRevokeResponse = _reflection.GeneratedProtocolMessageType('MsgRevokeResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGREVOKERESPONSE,
  '__module__' : 'cosmos.authz.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.authz.v1beta1.MsgRevokeResponse)
  })
_sym_db.RegisterMessage(MsgRevokeResponse)


DESCRIPTOR._options = None
_MSGGRANT.fields_by_name['grant']._options = None
_MSGEXEC.fields_by_name['msgs']._options = None

_MSG = _descriptor.ServiceDescriptor(
  name='Msg',
  full_name='cosmos.authz.v1beta1.Msg',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=576,
  serialized_end=824,
  methods=[
  _descriptor.MethodDescriptor(
    name='Grant',
    full_name='cosmos.authz.v1beta1.Msg.Grant',
    index=0,
    containing_service=None,
    input_type=_MSGGRANT,
    output_type=_MSGGRANTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Exec',
    full_name='cosmos.authz.v1beta1.Msg.Exec',
    index=1,
    containing_service=None,
    input_type=_MSGEXEC,
    output_type=_MSGEXECRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Revoke',
    full_name='cosmos.authz.v1beta1.Msg.Revoke',
    index=2,
    containing_service=None,
    input_type=_MSGREVOKE,
    output_type=_MSGREVOKERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MSG)

DESCRIPTOR.services_by_name['Msg'] = _MSG

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Implementation of IBC Applications Transfer  interface using REST."""
from google.protobuf.json_format import Parse

from cosmpy.common.rest_client import RestClient
from cosmpy.ibc.core.connection.interface import IBCCoreConnection  # type: ignore
from cosmpy.protos.ibc.core.connection.v1.query_pb2 import (
    QueryClientConnectionsRequest,
    QueryClientConnectionsResponse,
    QueryConnectionClientStateRequest,
    QueryConnectionClientStateResponse,
    QueryConnectionConsensusStateRequest,
    QueryConnectionConsensusStateResponse,
    QueryConnectionRequest,
    QueryConnectionResponse,
    QueryConnectionsRequest,
    QueryConnectionsResponse,
)


class IBCCoreConnectionRestClient(IBCCoreConnection):
    """IBC Core Connection REST client."""

    API_URL = "/ibc/core/connection/v1beta1"

    def __init__(self, rest_api: RestClient) -> None:
        """
        Initialize.

        :param rest_api: RestClient api
        """
        self._rest_api = rest_api

    def Connection(self, request: QueryConnectionRequest) -> QueryConnectionResponse:
        """
        Connection queries an IBC connection end.

        :param request: QueryConnectionRequest
        :return: QueryConnectionResponse
        """
        json_response = self._rest_api.get(
            f"{self.API_URL}/connections/{request.connection_id}"
        )
        return Parse(json_response, QueryConnectionResponse())

    def Connections(self, request: QueryConnectionsRequest) -> QueryConnectionsResponse:
        """
        Connections queries all the IBC connections of a chain.

        :param request: QueryConnectionsRequest
        :return: QueryConnectionsResponse
        """
        json_response = self._rest_api.get(f"{self.API_URL}/connections", request)
        return Parse(json_response, QueryConnectionsResponse())

    def ClientConnections(
        self, request: QueryClientConnectionsRequest
    ) -> QueryClientConnectionsResponse:
        """
        ClientConnections queries the connection paths associated with a client state.

        :param request: QueryClientConnectionsRequest
        :return: QueryClientConnectionsResponse
        """
        json_response = self._rest_api.get(
            f"{self.API_URL}/client_connections/{request.client_id}"
        )
        return Parse(json_response, QueryClientConnectionsResponse())

    def ConnectionClientState(
        self, request: QueryConnectionClientStateRequest
    ) -> QueryConnectionClientStateResponse:
        """
        ConnectionClientState queries the client state associated with the connection.

        :param request: QueryConnectionClientStateRequest
        :return: QueryConnectionClientStateResponse
        """
        json_response = self._rest_api.get(
            f"{self.API_URL}/connections/{request.connection_id}/client_state"
        )
        return Parse(json_response, QueryConnectionClientStateResponse())

    def ConnectionConsensusState(
        self, request: QueryConnectionConsensusStateRequest
    ) -> QueryConnectionConsensusStateResponse:
        """
        ConnectionConsensusState queries the consensus state associated with the connection.

        :param request: QueryConnectionConsensusStateRequest
        :return: QueryConnectionConsensusStateResponse
        """
        json_response = self._rest_api.get(
            f"{self.API_URL}/connections/{request.connection_id}/consensus_state/revision/{request.revision_number}/height/{request.revision_height}"
        )
        return Parse(json_response, QueryConnectionConsensusStateResponse())



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

from cosmpy.aerial.contract import (
    create_cosmwasm_execute_msg,
    create_cosmwasm_instantiate_msg,
)
from cosmpy.crypto.address import Address


def test_create_instantiate_msg():
    sender = Address("fetch1r3d4azhlak4w00c5n02t9l35a3n6462vrnunel")
    msg = create_cosmwasm_instantiate_msg(
        1, {}, "init-label", sender, funds="10atestfet", admin_address=sender
    )

    assert msg.sender == str(sender)
    assert msg.code_id == 1
    assert msg.msg == b"{}"
    assert msg.label == "init-label"
    assert msg.admin == str(sender)
    assert len(msg.funds) == 1
    assert msg.funds[0].denom == "atestfet"
    assert msg.funds[0].amount == "10"


def test_create_execute_msg():
    sender = Address("fetch1r3d4azhlak4w00c5n02t9l35a3n6462vrnunel")
    contract = Address("fetch1faucet4p2h432pxlh9ez8jfcl9jyr2ndlx2992")

    msg = create_cosmwasm_execute_msg(
        sender, contract, {}, funds="15atestfet,42another"
    )

    assert msg.sender == str(sender)
    assert msg.contract == str(contract)
    assert msg.msg == b"{}"
    assert len(msg.funds) == 2
    assert msg.funds[0].denom == "atestfet"
    assert msg.funds[0].amount == "15"
    assert msg.funds[1].denom == "another"
    assert msg.funds[1].amount == "42"



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Implementation of Params interface using REST."""

from google.protobuf.json_format import Parse

from cosmpy.common.rest_client import RestClient
from cosmpy.params.interface import Params
from cosmpy.protos.cosmos.params.v1beta1.query_pb2 import (
    QueryParamsRequest,
    QueryParamsResponse,
)


class ParamsRestClient(Params):
    """Params REST client."""

    API_URL = "/cosmos/params/v1beta1"

    def __init__(self, rest_api: RestClient) -> None:
        """
        Initialize.

        :param rest_api: RestClient api
        """
        self._rest_api = rest_api

    def Params(self, request: QueryParamsRequest) -> QueryParamsResponse:
        """
        Params queries a specific Cosmos SDK parameter.

        :param request: QueryParamsRequest
        :return: QueryParamsResponse
        """
        json_response = self._rest_api.get(f"{self.API_URL}/params", request)
        return Parse(json_response, QueryParamsResponse())



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

import pytest

from cosmpy.aerial.urls import ParsedUrl, Protocol, parse_url


@pytest.mark.parametrize(
    "input_url,parse_result",
    [
        (
            "grpc+https://foo.bar.baz",
            ParsedUrl(
                hostname="foo.bar.baz", port=443, secure=True, protocol=Protocol.GRPC
            ),
        ),
        (
            "grpc+https://foo.bar.baz:8000",
            ParsedUrl(
                hostname="foo.bar.baz", port=8000, secure=True, protocol=Protocol.GRPC
            ),
        ),
        (
            "grpc+http://foo.bar.baz",
            ParsedUrl(
                hostname="foo.bar.baz", port=80, secure=False, protocol=Protocol.GRPC
            ),
        ),
        (
            "grpc+http://foo.bar.baz:8000",
            ParsedUrl(
                hostname="foo.bar.baz", port=8000, secure=False, protocol=Protocol.GRPC
            ),
        ),
        (
            "rest+https://foo.bar.baz",
            ParsedUrl(
                hostname="foo.bar.baz", port=443, secure=True, protocol=Protocol.REST
            ),
        ),
        (
            "rest+https://foo.bar.baz:8000",
            ParsedUrl(
                hostname="foo.bar.baz", port=8000, secure=True, protocol=Protocol.REST
            ),
        ),
        (
            "rest+http://foo.bar.baz",
            ParsedUrl(
                hostname="foo.bar.baz", port=80, secure=False, protocol=Protocol.REST
            ),
        ),
        (
            "rest+http://foo.bar.baz:8000",
            ParsedUrl(
                hostname="foo.bar.baz", port=8000, secure=False, protocol=Protocol.REST
            ),
        ),
    ],
)
def test_parsing_urls(input_url, parse_result):
    assert parse_url(input_url) == parse_result



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Implementation of IBC Applications Transfer  interface using REST."""
from google.protobuf.json_format import Parse

from cosmpy.common.rest_client import RestClient
from cosmpy.ibc.applications.transfer.interface import (  # type: ignore
    IBCApplicationsTransfer,
)
from cosmpy.protos.ibc.applications.transfer.v1.query_pb2 import (
    QueryDenomTraceRequest,
    QueryDenomTraceResponse,
    QueryDenomTracesRequest,
    QueryDenomTracesResponse,
    QueryParamsRequest,
    QueryParamsResponse,
)


class IBCApplicationsTransferRestClient(IBCApplicationsTransfer):
    """IBC Applications Transfer REST client."""

    API_URL = "/ibc/applications/transfer/v1beta1"

    def __init__(self, rest_api: RestClient) -> None:
        """
        Initialize.

        :param rest_api: RestClient api
        """
        self._rest_api = rest_api

    def DenomTrace(self, request: QueryDenomTraceRequest) -> QueryDenomTraceResponse:
        """
        DenomTrace queries a denomination trace information.

        :param request: QueryDenomTraceRequest
        :return: QueryDenomTraceResponse
        """
        json_response = self._rest_api.get(
            f"{self.API_URL}/denom_traces/{request.hash}"
        )
        return Parse(json_response, QueryDenomTraceResponse())

    def DenomTraces(self, request: QueryDenomTracesRequest) -> QueryDenomTracesResponse:
        """
        DenomTraces queries all denomination traces.

        :param request: QueryDenomTracesRequest
        :return: QueryDenomTracesResponse
        """
        json_response = self._rest_api.get(f"{self.API_URL}/denom_traces", request)
        return Parse(json_response, QueryDenomTracesResponse())

    def Params(self, request: QueryParamsRequest) -> QueryParamsResponse:
        """
        Params queries all parameters of the ibc-transfer module.

        :param request: QueryParamsRequest
        :return: QueryParamsResponse
        """
        json_response = self._rest_api.get(f"{self.API_URL}/params")
        return Parse(json_response, QueryParamsResponse())



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
import gzip
import json
from typing import Any, Optional

from cosmpy.aerial.coins import parse_coins
from cosmpy.crypto.address import Address
from cosmpy.protos.cosmwasm.wasm.v1.tx_pb2 import (
    MsgExecuteContract,
    MsgInstantiateContract,
    MsgStoreCode,
)


def create_cosmwasm_store_code_msg(
    contract_path: str, sender_address: Address
) -> MsgStoreCode:
    with open(contract_path, "rb") as contract_file:
        wasm_byte_code = gzip.compress(contract_file.read(), 9)

    msg = MsgStoreCode(
        sender=str(sender_address),
        wasm_byte_code=wasm_byte_code,
    )

    return msg


def create_cosmwasm_instantiate_msg(
    code_id: int,
    args: Any,
    label: str,
    sender_address: Address,
    funds: Optional[str] = None,
    admin_address: Optional[Address] = None,
) -> MsgInstantiateContract:
    msg = MsgInstantiateContract(
        sender=str(sender_address),
        code_id=code_id,
        msg=json.dumps(args).encode("UTF8"),
        label=label,
    )

    if funds is not None:
        msg.funds.extend(parse_coins(funds))
    if admin_address is not None:
        msg.admin = str(admin_address)  # noqa

    return msg


def create_cosmwasm_execute_msg(
    sender_address: Address,
    contract_address: Address,
    args: Any,
    funds: Optional[str] = None,
) -> MsgExecuteContract:
    msg = MsgExecuteContract(
        sender=str(sender_address),
        contract=str(contract_address),
        msg=json.dumps(args).encode("UTF8"),
    )
    if funds is not None:
        msg.funds.extend(parse_coins(funds))

    return msg



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Tests for REST implementation of IBC Applications Transfer."""
import json
from typing import Dict, Tuple
from unittest import TestCase

from google.protobuf.json_format import ParseDict

from cosmpy.ibc.applications.transfer.rest_client import (  # type: ignore
    IBCApplicationsTransferRestClient,
)
from cosmpy.protos.ibc.applications.transfer.v1.query_pb2 import (
    QueryDenomTraceRequest,
    QueryDenomTraceResponse,
    QueryDenomTracesRequest,
    QueryDenomTracesResponse,
    QueryParamsRequest,
    QueryParamsResponse,
)
from tests.helpers import MockRestClient


class IBCApplicationsTransferRestClientTestCase(TestCase):
    """Test case for IBCApplicationsTransferRestClient class."""

    REST_CLIENT = IBCApplicationsTransferRestClient

    def make_clients(
        self, response_content: Dict
    ) -> Tuple[MockRestClient, IBCApplicationsTransferRestClient]:
        """
        Make  mock client and rest client api for specific content.

        :param response_content: dict
        :return: rest client instance
        """
        mock_client = MockRestClient(json.dumps(response_content).encode("utf-8"))
        rest_client = self.REST_CLIENT(mock_client)
        return mock_client, rest_client

    def test_DenomTrace(self):
        """Test DenomTrace method."""
        content = {"denom_trace": {"path": "string", "base_denom": "string"}}
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryDenomTraceResponse())

        assert (
            rest_client.DenomTrace(QueryDenomTraceRequest(hash="hash"))
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/applications/transfer/v1beta1/denom_traces/hash"
        )

    def test_DenomTraces(self):
        """Test DenomTraces method."""
        content = {
            "denom_traces": [{"path": "string", "base_denom": "string"}],
            "pagination": {"next_key": "string", "total": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryDenomTracesResponse())

        assert rest_client.DenomTraces(QueryDenomTracesRequest()) == expected_response
        assert (
            mock_client.last_base_url
            == "/ibc/applications/transfer/v1beta1/denom_traces"
        )

    def test_Params(self):
        """Test Params method."""
        content = {}
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryParamsResponse())

        assert rest_client.Params(QueryParamsRequest()) == expected_response
        assert mock_client.last_base_url == "/ibc/applications/transfer/v1beta1/params"



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

import argparse
import sys

from cosmpy.aerial.client import LedgerClient
from cosmpy.aerial.config import NetworkConfig
from cosmpy.aerial.contract import LedgerContract, create_cosmwasm_execute_msg
from cosmpy.aerial.tx import SigningCfg, Transaction
from cosmpy.aerial.wallet import LocalWallet
from cosmpy.crypto.address import Address
from cosmpy.crypto.keypairs import PrivateKey

TOKEN_ID_1 = "680564733841876926926749214863536422912"
TOKEN_ID_2 = "680564733841876926926749214863536422913"


def _parse_commandline():
    parser = argparse.ArgumentParser()
    parser.add_argument("contract_path", help="The path to the cosmwasm 1155 contract")
    parser.add_argument(
        "contract_address",
        nargs="?",
        type=Address,
        help="The address of the contract is already deployed",
    )
    return parser.parse_args()


def main():
    args = _parse_commandline()

    alice = LocalWallet(PrivateKey("X2Tv0Ok3RN2yi9GhWjLUX7RIfX5go9Wu+fwoJlqK2Og="))
    bob = LocalWallet(PrivateKey("p0h0sYImB4xGq3Zz+xfIrY4QR6CPqeNg8w6X3NUWLe4="))

    client = LedgerClient(NetworkConfig.fetchai_stable_testnet())

    # check to see if all the clients have enough funds
    alice_balance = client.query_bank_balance(alice.address())
    bob_balance = client.query_bank_balance(bob.address())
    print(f"Alice balance {alice_balance}")
    print(f"Bob   balance {bob_balance}")

    if alice_balance < (10**18):
        print(
            f"Alice has insufficient balance to complete operations please funds alice account: {alice.address()}"
        )
        sys.exit(1)

    if bob_balance < (10**18):
        print(
            f"Bob has insufficient balance to complete operations please funds bob account: {bob.address()}"
        )
        sys.exit(1)

    contract = LedgerContract(args.contract_path, client, address=args.contract_address)

    # Step 1. Deploy the contract and setup the initial balances as required
    if contract.address is None:
        print("Deploying contract...")
        contract.deploy({}, alice)
        print(f"Contract deployed at {contract.address}")

        create_batch_msg = {
            "create_batch": {
                "item_owner": str(alice.address()),
                "tokens": [
                    {
                        "id": TOKEN_ID_1,
                        "path": "some_path",
                    },
                    {
                        "id": TOKEN_ID_2,
                        "path": "some_path",
                    },
                ],
            }
        }

        print("Creating tokens...")
        tx = contract.execute(create_batch_msg, alice).wait_to_complete()
        print(f"Created tokens (tx: {tx.tx_hash})")

        # Create Alice's token
        mint_single_msg = {
            "mint_single": {
                "to_address": str(alice.address()),
                "id": TOKEN_ID_1,
                "supply": "2000",
                "data": "some_data",
            },
        }

        print(f"Minting tokens (id: {TOKEN_ID_1})...")
        tx = contract.execute(mint_single_msg, alice).wait_to_complete()
        print(f"Minted tokens (id: {TOKEN_ID_1} tx: {tx.tx_hash})")

        # Create Bob's token
        mint_single_msg = {
            "mint_single": {
                "to_address": str(bob.address()),
                "id": TOKEN_ID_2,
                "supply": "2000",
                "data": "some_data",
            },
        }

        print(f"Minting tokens (id: {TOKEN_ID_1})...")
        tx = contract.execute(mint_single_msg, alice).wait_to_complete()
        print(f"Minted tokens (id: {TOKEN_ID_1} tx: {tx.tx_hash})")

    # Step 2. Query what the current balance state is
    result = contract.query(
        {
            "balance_batch": {
                "addresses": [
                    {
                        "address": str(bob.address()),
                        "id": TOKEN_ID_1,
                    },
                    {
                        "address": str(alice.address()),
                        "id": TOKEN_ID_2,
                    },
                ]
            }
        }
    )
    print(f'Alice has {result["balances"][1]} of Bob\'s tokens')
    print(f'Bob has {result["balances"][0]} of Alice\'s tokens')

    # Step 3. Perform the atomic swap using the transaction builder and the client

    # Perform an atomic swap. Since this is a specialised operation we need to be able to manually
    # build up the contents of the transaction
    tx = Transaction()

    # add the first message which is instructing the transfer of Alice's token to Bob
    tx.add_message(
        create_cosmwasm_execute_msg(
            alice.address(),
            contract.address,
            {
                "transfer_single": {
                    "operator": str(alice.address()),
                    "from_address": str(alice.address()),
                    "to_address": str(bob.address()),
                    "id": TOKEN_ID_1,
                    "value": "1",
                },
            },
        )
    )

    # add the second message which is instructing the transfer of Bob's token to Alice
    tx.add_message(
        create_cosmwasm_execute_msg(
            bob.address(),
            contract.address,
            {
                "transfer_single": {
                    "operator": str(bob.address()),
                    "from_address": str(bob.address()),
                    "to_address": str(alice.address()),
                    "id": TOKEN_ID_2,
                    "value": "1",
                },
            },
        )
    )

    # lookup the account information for each for Alice and Bob
    alice_account = client.query_account(alice.address())
    bob_account = client.query_account(bob.address())

    # gas and fee estimation
    gas_limit = 300000
    fee = client.estimate_fee_from_gas(gas_limit)

    # seal the transaction: this stops all further updates to the transactions and this is the step
    # where additional metadata is added like gas, fees and the individual signers sequence numbers
    tx.seal(
        [
            SigningCfg.direct(alice.public_key(), alice_account.sequence),
            SigningCfg.direct(bob.public_key(), bob_account.sequence),
        ],
        fee,
        gas_limit,
    )

    # both Alice and Bob sign the transaction
    tx.sign(alice.signer(), client.network_config.chain_id, alice_account.number)
    tx.sign(bob.signer(), client.network_config.chain_id, bob_account.number)

    # all done!
    tx.complete()

    print("Executing atomic swap...")
    tx = client.broadcast_tx(tx).wait_to_complete()
    print(f"Executing atomic swap...complete (tx: {tx.tx_hash})")

    # Step 4. Query what the new balance state is
    result = contract.query(
        {
            "balance_batch": {
                "addresses": [
                    {
                        "address": str(bob.address()),
                        "id": TOKEN_ID_1,
                    },
                    {
                        "address": str(alice.address()),
                        "id": TOKEN_ID_2,
                    },
                ]
            }
        }
    )
    print(f'Alice has {result["balances"][1]} of Bob\'s tokens')
    print(f'Bob has {result["balances"][0]} of Alice\'s tokens')


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Implementation of Evidence interface using REST."""

from google.protobuf.json_format import Parse

from cosmpy.common.rest_client import RestClient
from cosmpy.evidence.interface import Evidence
from cosmpy.protos.cosmos.evidence.v1beta1.query_pb2 import (
    QueryAllEvidenceRequest,
    QueryAllEvidenceResponse,
    QueryEvidenceRequest,
    QueryEvidenceResponse,
)


class EvidenceRestClient(Evidence):
    """Evidence REST client."""

    API_URL = "/cosmos/evidence/v1beta1"

    def __init__(self, rest_api: RestClient) -> None:
        """
        Initialize.

        :param rest_api: RestClient api
        """
        self._rest_api = rest_api

    def Evidence(self, request: QueryEvidenceRequest) -> QueryEvidenceResponse:
        """
        Evidence queries evidence based on evidence hash.

        :param request: QueryEvidenceRequest

        :return: QueryEvidenceResponse
        """

        json_response = self._rest_api.get(
            f"{self.API_URL}/evidence/{request.evidence_hash}",
        )
        return Parse(json_response, QueryEvidenceResponse())

    def AllEvidence(self, request: QueryAllEvidenceRequest) -> QueryAllEvidenceResponse:
        """
        AllEvidence queries all evidence.

        :param request: QueryAllEvidenceRequest

        :return: QueryAllEvidenceResponse
        """

        json_response = self._rest_api.get(f"{self.API_URL}/evidence", request)
        return Parse(json_response, QueryAllEvidenceResponse())



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Tests for REST implementation of Gov."""
import json
from typing import Dict, Tuple
from unittest import TestCase

from google.protobuf.json_format import ParseDict

from cosmpy.gov.rest_client import GovRestClient
from cosmpy.protos.cosmos.gov.v1beta1.query_pb2 import (
    QueryDepositRequest,
    QueryDepositResponse,
    QueryDepositsRequest,
    QueryDepositsResponse,
    QueryParamsRequest,
    QueryParamsResponse,
    QueryProposalRequest,
    QueryProposalResponse,
    QueryProposalsRequest,
    QueryProposalsResponse,
    QueryTallyResultRequest,
    QueryTallyResultResponse,
    QueryVoteRequest,
    QueryVoteResponse,
    QueryVotesRequest,
    QueryVotesResponse,
)
from tests.helpers import MockRestClient


class GovRestClientTestCase(TestCase):
    """Test case for GovRestClient class."""

    REST_CLIENT = GovRestClient

    def make_clients(
        self, response_content: Dict
    ) -> Tuple[MockRestClient, GovRestClient]:
        """
        Make  mock client and rest client api for specific content.

        :param response_content: dict
        :return: rest client instance
        """
        mock_client = MockRestClient(json.dumps(response_content).encode("utf-8"))
        rest_client = self.REST_CLIENT(mock_client)
        return mock_client, rest_client

    def test_Proposal(self):
        """Test Proposal method."""
        content = {
            "proposal": {
                "proposal_id": "10",
                "status": "PROPOSAL_STATUS_UNSPECIFIED",
                "final_tally_result": {
                    "yes": "string",
                    "abstain": "string",
                    "no": "string",
                    "no_with_veto": "string",
                },
                "submit_time": "2022-02-21T13:03:00.840Z",
                "deposit_end_time": "2022-02-21T13:03:00.840Z",
                "total_deposit": [{"denom": "string", "amount": "string"}],
                "voting_start_time": "2022-02-21T13:03:00.840Z",
                "voting_end_time": "2022-02-21T13:03:00.840Z",
            }
        }
        mock_client, gov = self.make_clients(content)
        expected_response = ParseDict(content, QueryProposalResponse())

        assert gov.Proposal(QueryProposalRequest(proposal_id=1)) == expected_response
        assert mock_client.last_base_url == "/cosmos/gov/v1beta1/proposals/1"

    def test_Proposals(self):
        """Test Proposals method."""
        content = {
            "proposals": [
                {
                    "proposal_id": "10",
                    "status": "PROPOSAL_STATUS_UNSPECIFIED",
                    "final_tally_result": {
                        "yes": "string",
                        "abstain": "string",
                        "no": "string",
                        "no_with_veto": "string",
                    },
                    "submit_time": "2022-02-21T13:09:02.841Z",
                    "deposit_end_time": "2022-02-21T13:09:02.842Z",
                    "total_deposit": [{"denom": "string", "amount": "string"}],
                    "voting_start_time": "2022-02-21T13:09:02.842Z",
                    "voting_end_time": "2022-02-21T13:09:02.842Z",
                }
            ],
            "pagination": {"next_key": "string", "total": "1"},
        }
        mock_client, gov = self.make_clients(content)
        expected_response = ParseDict(content, QueryProposalsResponse())

        assert gov.Proposals(QueryProposalsRequest()) == expected_response
        assert mock_client.last_base_url == "/cosmos/gov/v1beta1/proposals/"

    def test_Vote(self):
        """Test Vote method."""
        content = {"vote": {"proposal_id": "10", "voter": "addr"}}
        mock_client, gov = self.make_clients(content)
        expected_response = ParseDict(content, QueryVoteResponse())

        assert (
            gov.Vote(QueryVoteRequest(proposal_id=10, voter="addr"))
            == expected_response
        )
        assert (
            mock_client.last_base_url == "/cosmos/gov/v1beta1/proposals/10/votes/addr"
        )

    def test_Votes(self):
        """Test Votes method."""
        content = {
            "votes": [{"proposal_id": "1", "voter": "string"}],
            "pagination": {"next_key": "string", "total": "1"},
        }
        mock_client, gov = self.make_clients(content)
        expected_response = ParseDict(content, QueryVotesResponse())

        assert gov.Votes(QueryVotesRequest(proposal_id=1)) == expected_response
        assert mock_client.last_base_url == "/cosmos/gov/v1beta1/proposals/1/votes/"

    def test_Params(self):
        """Test Params method."""
        content = {
            "voting_params": {"voting_period": "10s"},
            "deposit_params": {
                "min_deposit": [{"denom": "string", "amount": "string"}],
                "max_deposit_period": "10s",
            },
            "tally_params": {
                "quorum": "string",
                "threshold": "string",
                "veto_threshold": "string",
            },
        }
        mock_client, gov = self.make_clients(content)
        expected_response = ParseDict(content, QueryParamsResponse())

        assert gov.Params(QueryParamsRequest(params_type="some")) == expected_response
        assert mock_client.last_base_url == "/cosmos/gov/v1beta1/params/some"

    def test_Deposit(self):
        """Test Deposit method."""
        content = {
            "deposit": {
                "proposal_id": "10",
                "depositor": "string",
                "amount": [{"denom": "string", "amount": "10"}],
            }
        }
        mock_client, gov = self.make_clients(content)
        expected_response = ParseDict(content, QueryDepositResponse())

        assert (
            gov.Deposit(QueryDepositRequest(proposal_id=10, depositor="depositor"))
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/cosmos/gov/v1beta1/proposals/10/deposits/depositor"
        )

    def test_Deposits(self):
        """Test Deposits method."""
        content = {
            "deposits": [
                {
                    "proposal_id": "10",
                    "depositor": "string",
                    "amount": [{"denom": "string", "amount": "10"}],
                }
            ],
            "pagination": {"next_key": "string", "total": "1"},
        }
        mock_client, gov = self.make_clients(content)
        expected_response = ParseDict(content, QueryDepositsResponse())

        assert gov.Deposits(QueryDepositsRequest(proposal_id=1)) == expected_response
        assert mock_client.last_base_url == "/cosmos/gov/v1beta1/proposals/1/deposits/"

    def test_TallyResult(self):
        """Test TallyResult method."""
        content = {
            "tally": {
                "yes": "string",
                "abstain": "string",
                "no": "string",
                "no_with_veto": "string",
            }
        }
        mock_client, gov = self.make_clients(content)
        expected_response = ParseDict(content, QueryTallyResultResponse())

        assert (
            gov.TallyResult(QueryTallyResultRequest(proposal_id=10))
            == expected_response
        )
        assert mock_client.last_base_url == "/cosmos/gov/v1beta1/proposals/10/tally"



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Tests for REST implementation of IBC Core Connection."""
import json
from typing import Dict, Tuple
from unittest import TestCase

from google.protobuf.json_format import ParseDict
from google.protobuf.wrappers_pb2 import Int32Value  # noqa # needed for protobuf decode

from cosmpy.ibc.core.connection.rest_client import (  # type: ignore
    IBCCoreConnectionRestClient,
)
from cosmpy.protos.ibc.core.connection.v1.query_pb2 import (
    QueryClientConnectionsRequest,
    QueryClientConnectionsResponse,
    QueryConnectionClientStateRequest,
    QueryConnectionClientStateResponse,
    QueryConnectionConsensusStateRequest,
    QueryConnectionConsensusStateResponse,
    QueryConnectionRequest,
    QueryConnectionResponse,
    QueryConnectionsRequest,
    QueryConnectionsResponse,
)
from tests.helpers import MockRestClient

TYPE = {
    "@type": "type.googleapis.com/google.protobuf.Int32Value",
    "value": "42",
}


class IBCCoreConnectionRestClientTestCase(TestCase):
    """Test case for IBCCoreConnectionRestClient class."""

    REST_CLIENT = IBCCoreConnectionRestClient

    def make_clients(
        self, response_content: Dict
    ) -> Tuple[MockRestClient, IBCCoreConnectionRestClient]:
        """
        Make  mock client and rest client api for specific content.

        :param response_content: dict
        :return: rest client instance
        """
        mock_client = MockRestClient(json.dumps(response_content).encode("utf-8"))
        rest_client = self.REST_CLIENT(mock_client)
        return mock_client, rest_client

    def test_Connection(self):
        """Test Connection method."""
        content = {
            "connection": {
                "client_id": "string",
                "versions": [{"identifier": "string", "features": ["string"]}],
                "state": "STATE_UNINITIALIZED_UNSPECIFIED",
                "counterparty": {
                    "client_id": "1",
                    "connection_id": "1",
                    "prefix": {"key_prefix": "string"},
                },
                "delay_period": "1",
            },
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryConnectionResponse())

        assert (
            rest_client.Connection(
                QueryConnectionRequest(connection_id="connection_id")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/connection/v1beta1/connections/connection_id"
        )

    def test_Connections(self):
        """Test Connections method."""
        content = {
            "connections": [
                {
                    "id": "1",
                    "client_id": "1",
                    "versions": [{"identifier": "string", "features": ["string"]}],
                    "state": "STATE_UNINITIALIZED_UNSPECIFIED",
                    "counterparty": {
                        "client_id": "1",
                        "connection_id": "1",
                        "prefix": {"key_prefix": "string"},
                    },
                    "delay_period": "1",
                }
            ],
            "pagination": {"next_key": "string", "total": "1"},
            "height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryConnectionsResponse())

        assert rest_client.Connections(QueryConnectionsRequest()) == expected_response
        assert mock_client.last_base_url == "/ibc/core/connection/v1beta1/connections"

    def test_ClientConnections(self):
        """Test ClientConnections method."""
        content = {
            "connection_paths": ["string"],
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryClientConnectionsResponse())

        assert (
            rest_client.ClientConnections(
                QueryClientConnectionsRequest(client_id="111")
            )
            == expected_response
        )
        assert [mock_client.last_base_url] == [
            "/ibc/core/connection/v1beta1/client_connections/111"
        ]

    def test_ConnectionClientState(self):
        """Test ConnectionClientState method."""
        content = {
            "identified_client_state": {
                "client_id": "string",
                "client_state": TYPE,
            },
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryConnectionClientStateResponse())

        assert (
            rest_client.ConnectionClientState(
                QueryConnectionClientStateRequest(connection_id="connection_id")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/connection/v1beta1/connections/connection_id/client_state"
        )

    def test_ConnectionConsensusState(self):
        """Test ConnectionConsensusState method."""
        content = {
            "consensus_state": TYPE,
            "client_id": "string",
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryConnectionConsensusStateResponse())

        assert (
            rest_client.ConnectionConsensusState(
                QueryConnectionConsensusStateRequest(
                    revision_height=1,
                    revision_number=1,
                    connection_id="connection_id",
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/connection/v1beta1/connections/connection_id/consensus_state/revision/1/height/1"
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Hash functions of Crypto package."""

import hashlib

from _hashlib import HASH  # type: ignore  # pylint: disable=no-name-in-module

# Detect if ripemd160 can actually be used in the system. Querying `hashlib.algorithms_available`
# does not mean much and will fail on 22.04 LTS
try:
    hashlib.new("ripemd160")
    _ripemd160_present = True
except ValueError:
    _ripemd160_present = False


def sha256(contents: bytes) -> bytes:
    """
    Get sha256 hash.

    :param contents: bytes contents.

    :return: bytes sha256 hash.
    """
    h: HASH = hashlib.sha256()
    h.update(contents)
    return h.digest()


def _ripemd160_stdlib(contents: bytes) -> bytes:
    h: HASH = hashlib.new("ripemd160")
    h.update(contents)
    return h.digest()


def _ripemd160_mbedtls(contents: bytes) -> bytes:
    from mbedtls import hashlib as alt_hashlib

    h: HASH = alt_hashlib.new("ripemd160")
    h.update(contents)
    return h.digest()


def ripemd160(contents: bytes) -> bytes:
    """
    Get ripemd160 hash.

    :param contents: bytes contents.

    :return: bytes ripemd160 hash.
    """

    # Check if we need to use the fallback hashlib
    if not _ripemd160_present:
        return _ripemd160_mbedtls(contents)

    # Prefer the stdlib implementation
    return _ripemd160_stdlib(contents)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Integration tests for contract functions."""
from pathlib import Path

import pytest

from cosmpy.aerial.client import LedgerClient
from cosmpy.aerial.config import NetworkConfig
from cosmpy.aerial.contract import LedgerContract
from cosmpy.aerial.faucet import FaucetApi
from cosmpy.aerial.wallet import LocalWallet

CONTRACT_PATH = Path(__file__).parent / "../../contracts/simple.wasm"


@pytest.mark.integration
def test_contract():
    """Test simple contract deploy execute and query."""
    wallet = LocalWallet.generate()
    faucet_api = FaucetApi(NetworkConfig.fetchai_stable_testnet())
    faucet_api.get_wealth(wallet.address())
    ledger = LedgerClient(NetworkConfig.fetchai_stable_testnet())
    contract = LedgerContract(CONTRACT_PATH, ledger)
    contract_address = contract.deploy({}, wallet)
    assert contract_address
    result = contract.query({"get": {"owner": str(wallet.address())}})

    assert not result["exists"]
    assert not result["value"]

    value = "foobar"
    contract.execute({"set": {"value": value}}, wallet).wait_to_complete()
    result = contract.query({"get": {"owner": str(wallet.address())}})

    assert result["exists"]
    assert result["value"] == value


@pytest.mark.integration
def test_deployed_contract():
    """Test interaction with already deployed contract."""
    wallet = LocalWallet.generate()
    faucet_api = FaucetApi(NetworkConfig.fetchai_stable_testnet())
    faucet_api.get_wealth(wallet.address())
    ledger = LedgerClient(NetworkConfig.fetchai_stable_testnet())
    contract = LedgerContract(CONTRACT_PATH, ledger)
    contract_address = contract.deploy({}, wallet)
    assert contract_address

    deployed_contract = LedgerContract(None, ledger, contract_address)

    result = deployed_contract.query({"get": {"owner": str(wallet.address())}})

    assert not result["exists"]
    assert not result["value"]

    value = "foobar"
    deployed_contract.execute({"set": {"value": value}}, wallet).wait_to_complete()
    result = deployed_contract.query({"get": {"owner": str(wallet.address())}})

    assert result["exists"]
    assert result["value"] == value


if __name__ == "__main__":
    pytest.main([__file__])



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/crypto/ed25519/keys.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cosmos/crypto/ed25519/keys.proto',
  package='cosmos.crypto.ed25519',
  syntax='proto3',
  serialized_options=b'Z0github.com/cosmos/cosmos-sdk/crypto/keys/ed25519',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n cosmos/crypto/ed25519/keys.proto\x12\x15\x63osmos.crypto.ed25519\x1a\x14gogoproto/gogo.proto\"9\n\x06PubKey\x12)\n\x03key\x18\x01 \x01(\x0c\x42\x1c\xfa\xde\x1f\x18\x63rypto/ed25519.PublicKey:\x04\x98\xa0\x1f\x00\"5\n\x07PrivKey\x12*\n\x03key\x18\x01 \x01(\x0c\x42\x1d\xfa\xde\x1f\x19\x63rypto/ed25519.PrivateKeyB2Z0github.com/cosmos/cosmos-sdk/crypto/keys/ed25519b\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,])




_PUBKEY = _descriptor.Descriptor(
  name='PubKey',
  full_name='cosmos.crypto.ed25519.PubKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='cosmos.crypto.ed25519.PubKey.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372\336\037\030crypto/ed25519.PublicKey', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\230\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=138,
)


_PRIVKEY = _descriptor.Descriptor(
  name='PrivKey',
  full_name='cosmos.crypto.ed25519.PrivKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='cosmos.crypto.ed25519.PrivKey.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372\336\037\031crypto/ed25519.PrivateKey', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=140,
  serialized_end=193,
)

DESCRIPTOR.message_types_by_name['PubKey'] = _PUBKEY
DESCRIPTOR.message_types_by_name['PrivKey'] = _PRIVKEY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PubKey = _reflection.GeneratedProtocolMessageType('PubKey', (_message.Message,), {
  'DESCRIPTOR' : _PUBKEY,
  '__module__' : 'cosmos.crypto.ed25519.keys_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.crypto.ed25519.PubKey)
  })
_sym_db.RegisterMessage(PubKey)

PrivKey = _reflection.GeneratedProtocolMessageType('PrivKey', (_message.Message,), {
  'DESCRIPTOR' : _PRIVKEY,
  '__module__' : 'cosmos.crypto.ed25519.keys_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.crypto.ed25519.PrivKey)
  })
_sym_db.RegisterMessage(PrivKey)


DESCRIPTOR._options = None
_PUBKEY.fields_by_name['key']._options = None
_PUBKEY._options = None
_PRIVKEY.fields_by_name['key']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
import pytest

from cosmpy.aerial.coins import parse_coins
from cosmpy.protos.cosmos.base.v1beta1.coin_pb2 import Coin


@pytest.mark.parametrize(
    "input_coins,expected_result",
    [
        ("", []),
        ("          ", []),
        ("50000atestfet", [Coin(amount="50000", denom="atestfet")]),
        (
            "50000atestfet,     200foobar",
            [
                Coin(amount="50000", denom="atestfet"),
                Coin(amount="200", denom="foobar"),
            ],
        ),
    ],
)
def test_parsing_coins(input_coins, expected_result):
    assert parse_coins(input_coins) == expected_result



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ibc/applications/transfer/v2/packet.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ibc/applications/transfer/v2/packet.proto',
  package='ibc.applications.transfer.v2',
  syntax='proto3',
  serialized_options=b'Z7github.com/cosmos/ibc-go/v2/modules/apps/transfer/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n)ibc/applications/transfer/v2/packet.proto\x12\x1cibc.applications.transfer.v2\"Z\n\x17\x46ungibleTokenPacketData\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\x12\x0e\n\x06\x61mount\x18\x02 \x01(\t\x12\x0e\n\x06sender\x18\x03 \x01(\t\x12\x10\n\x08receiver\x18\x04 \x01(\tB9Z7github.com/cosmos/ibc-go/v2/modules/apps/transfer/typesb\x06proto3'
)




_FUNGIBLETOKENPACKETDATA = _descriptor.Descriptor(
  name='FungibleTokenPacketData',
  full_name='ibc.applications.transfer.v2.FungibleTokenPacketData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='denom', full_name='ibc.applications.transfer.v2.FungibleTokenPacketData.denom', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='amount', full_name='ibc.applications.transfer.v2.FungibleTokenPacketData.amount', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sender', full_name='ibc.applications.transfer.v2.FungibleTokenPacketData.sender', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='receiver', full_name='ibc.applications.transfer.v2.FungibleTokenPacketData.receiver', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=75,
  serialized_end=165,
)

DESCRIPTOR.message_types_by_name['FungibleTokenPacketData'] = _FUNGIBLETOKENPACKETDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FungibleTokenPacketData = _reflection.GeneratedProtocolMessageType('FungibleTokenPacketData', (_message.Message,), {
  'DESCRIPTOR' : _FUNGIBLETOKENPACKETDATA,
  '__module__' : 'ibc.applications.transfer.v2.packet_pb2'
  # @@protoc_insertion_point(class_scope:ibc.applications.transfer.v2.FungibleTokenPacketData)
  })
_sym_db.RegisterMessage(FungibleTokenPacketData)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

import re
from typing import List

from cosmpy.protos.cosmos.base.v1beta1.coin_pb2 import Coin


def parse_coins(value: str) -> List[Coin]:
    coins = []

    parts = re.split(r",\s*", value)
    for part in parts:
        part = part.strip()
        if part == "":
            continue

        match = re.match(r"(\d+)(\w+)", part)
        if match is None:
            raise RuntimeError(f"Unable to parse value {part}")

        # extract out the groups
        amount, denom = match.groups()
        coins.append(Coin(amount=amount, denom=denom))

    return coins
